import { jsx as _jsx, Fragment as _Fragment, jsxs as _jsxs } from "@emotion/react/jsx-runtime";
/** @jsxImportSource @emotion/react */
import * as React from "react";
import PropTypes from "prop-types";
import { forwardRef, useRef, useState } from "react";
import { autoUpdate, size, useId, useDismiss, useFloating, useInteractions, useListNavigation, useRole, FloatingFocusManager, FloatingPortal } from "@floating-ui/react";
import { InputGroupContext, useActiveStyle, useSharedStyle } from "./Form";
const Item = forwardRef(({ children, active, index, ...rest }, ref) => {
    const id = useId();
    return (_jsx("div", { ref: ref, role: "option", id: id, "aria-selected": active, ...rest, style: {
            background: active ? "lightblue" : index % 2 === 0 ? "none" : "lavender",
            padding: 4,
            cursor: "default",
            ...rest.style
        }, children: children }));
});
/**解决<input>的局限性；
 * TextArea为基础的附加datalist; 因为input只能单一行的无法适应小屏幕大文本的输入； 由于<input 若用大size参数会有屏幕适应性问题！不建议用size设置尺寸！
 * onChange的差别！ props透传赋值的 前面的{...other} 被后面的同名字onChange所覆盖掉！！
 */
export const BlobInputList = ({ inputSize = "md", value, style, datalist = [], placeholder, onListChange, ...other }) => {
    const { bind, active } = useActiveStyle();
    const { baseStyles, inputSizes, activeBackground, errorStyles } = useSharedStyle();
    const { uid, error } = React.useContext(InputGroupContext);
    //下面才是列表赋能部分：
    const [open, setOpen] = useState(false);
    const [inputValue, setInputValue] = useState(value);
    const [activeIndex, setActiveIndex] = useState(null);
    React.useEffect(() => {
        setInputValue(value); //被外部修改的 可以同步更新吗？ [侦听]外部协作的输入变动！
    }, [value]);
    const listRef = useRef([]);
    const { x, y, reference, floating, strategy, context } = useFloating({
        whileElementsMounted: autoUpdate,
        open,
        onOpenChange: setOpen,
        middleware: [
            size({
                apply({ rects, availableHeight, elements }) {
                    Object.assign(elements.floating.style, {
                        width: `${rects.reference.width}px`,
                        maxHeight: `${availableHeight}px`
                    });
                },
                padding: 10
            })
        ]
    });
    // useLayoutEffect(() => {
    //     // IMPORTANT: When the floating element first opens, this effect runs when
    //     // the styles have **not yet** been applied to the element. A rAF ensures
    //     // we wait until the position is ready, and also runs before paint.
    //     // https://floating-ui.com/docs/react-dom#effects
    //     requestAnimationFrame(() => {
    //         if (activeIndex != null) {
    //             listRef.current[activeIndex]?.scrollIntoView({ block: "nearest" });
    //         }
    //     });
    // }, [activeIndex]);
    const { getReferenceProps, getFloatingProps, getItemProps } = useInteractions([
        useRole(context, { role: "listbox" }),
        useDismiss(context),
        useListNavigation(context, {
            listRef,
            activeIndex,
            onNavigate: setActiveIndex,
            virtual: true,
            loop: true
        })
    ]);
    function onChange(event) {
        const value = event.target.value;
        // console.log("商城组件 onChange的： value=",value);
        setInputValue(value);
        if (value) {
            setOpen(true);
            setActiveIndex(0);
        }
        else {
            setOpen(false);
        }
        onListChange(value);
    }
    const items = inputValue ? datalist.filter((item) => item.toLowerCase().includes(inputValue.toLowerCase()))
        :
            datalist; //全部可看
    return (_jsxs(_Fragment, { children: [_jsx("textarea", { className: "BlobInputList", id: uid, ...bind, css: [
                    baseStyles,
                    inputSizes[inputSize],
                    {
                        overflow: "auto",
                        resize: "vertical",
                    },
                    active && activeBackground,
                    error && errorStyles,
                    { ...style }
                ], ...other, ...getReferenceProps({
                    ref: reference,
                    onChange,
                    value: inputValue,
                    placeholder: placeholder,
                    "aria-autocomplete": "list",
                    onKeyDown(event) {
                        if (event.key === "Enter" &&
                            activeIndex != null &&
                            items[activeIndex]) {
                            setInputValue(items[activeIndex]);
                            setActiveIndex(null);
                            setOpen(false);
                        }
                    },
                    onPointerDown(event) {
                        setOpen(true);
                    },
                    style: {
                        ...style,
                    }
                }) }), _jsx(FloatingPortal, { children: open && (_jsx(FloatingFocusManager, { context: context, initialFocus: -1, visuallyHiddenDismiss: true, children: _jsx("div", { ...getFloatingProps({
                            ref: floating,
                            style: {
                                position: strategy,
                                left: x ?? 0,
                                top: y ?? 0,
                                background: "#eee",
                                color: "black",
                                overflowY: "auto"
                            }
                        }), children: items.map((item, index) => (_jsx(Item, { index: index, ...getItemProps({
                                key: item,
                                ref(node) {
                                    listRef.current[index] = node;
                                },
                                onClick() {
                                    setInputValue(item);
                                    setOpen(false);
                                    //refs.reference.current?.focus();
                                    onListChange(item);
                                },
                                style: {
                                    listStyleType: 'none',
                                    padding: 0,
                                    margin: 0,
                                },
                            }), active: activeIndex === index, children: item }))) }) })) })] }));
};
BlobInputList.propTypes = {
    inputSize: PropTypes.oneOf(["sm", "md", "lg"])
};
//# sourceMappingURL=BlobInputList.js.map