import { jsx as _jsx, jsxs as _jsxs } from "@emotion/react/jsx-runtime";
/** @jsxImportSource @emotion/react */
import { css } from "@emotion/react";
import * as React from "react";
import color from "color";
import PropTypes from "prop-types";
import { alpha, lighten } from "./Theme/colors";
import { Spinner } from "./Spinner";
import { useTheme } from "./Theme/Providers";
import { IconWrapper } from "./IconWrapper";
// import { useTouchable,  } from "./Hooks/touchable-hook";
import cx from "classnames";
import { safeBind } from "./Hooks/compose-bind";
// import {ForwardRefRenderFunction} from "react";
// import {ElementType} from "react";
import { usePressable } from "./Hooks/pressable-hook";
const getTextColor = (background, theme) => {
    return color(background).isDark() ? "white" : theme.modes.light.text.default;
};
export const getHeight = (size) => {
    if (size === "xs")
        return "25px";
    else if (size === "sm")
        return "30px";
    else if (size === "lg")
        return "48px";
    else if (size === "xl")
        return "60px";
    else
        return "40px";
};
const getPadding = (size, screen) => {
    if (screen === 'sm') {
        if (size === "xs")
            return "0 0.3rem";
        else if (size === "sm")
            return "0 0.4rem";
        else if (size === "lg")
            return "0 0.6rem";
        else if (size === "xl")
            return "0 0.7rem";
        else
            return "0 0.5rem";
    }
    else if (screen === 'lg') {
        if (size === "xs")
            return "0 0.6rem";
        else if (size === "sm")
            return "0 0.8rem";
        else if (size === "lg")
            return "0 1.2rem";
        else if (size === "xl")
            return "0 1.4rem";
        return "0 1rem";
    }
    else {
        if (size === "xs")
            return "0 0.5rem";
        else if (size === "sm")
            return "0 0.6rem";
        else if (size === "lg")
            return "0 1rem";
        else if (size === "xl")
            return "0 1.1rem";
        else
            return "0 0.8rem";
    }
};
const getFontSize = (size, theme) => {
    if (size === "xs")
        return theme.fontSizes[0];
    else if (size === "lg")
        return theme.fontSizes[2];
    else if (size === "sm")
        return theme.fontSizes[0];
    else if (size === "xl")
        return theme.fontSizes[3];
    else
        return theme.fontSizes[1];
};
const getDisplay = (block) => (block ? "flex" : "inline-flex");
function gradient(start, end) {
    return `linear-gradient(to top, ${start}, ${end})`;
}
function insetShadow(from, to) {
    return `inset 0 0 0 1px ${from}, inset 0 -1px 1px 0 ${to}`;
}
export function focusShadow(first, second, third) {
    return `0 0 0 3px ${first}, inset 0 0 0 1px ${second}, inset 0 -1px 1px 0 ${third}`;
}
export const buttonReset = css({
    textDecoration: "none",
    background: "none",
    whiteSpace: "nowrap",
    WebkitAppearance: "none",
    boxSizing: "border-box",
    textAlign: "center",
    WebkitFontSmoothing: "antialiased",
    border: "none",
    textRendering: "optimizeLegibility",
    userSelect: "none",
    WebkitTapHighlightColor: "transparent",
    cursor: "pointer",
    boxShadow: "none"
});
/**
 * Intent describes the intention of the button
 * -- eg., does it indicate danger?
 */
const getIntentStyles = (theme, intent, _hover, active) => {
    const dark = theme.colors.mode === "dark";
    const { palette } = theme.colors;
    const activeShadow = dark
        ? insetShadow(alpha(palette.gray.dark, 1), alpha(palette.gray.dark, 0.9))
        : insetShadow(alpha(palette.gray.dark, 0.5), alpha(palette.gray.dark, 0.5));
    switch (intent) {
        case "none": {
            if (dark)
                return css([
                    {
                        backgroundColor: theme.colors.scales.gray[5],
                        color: getTextColor(theme.colors.intent.none.base, theme),
                        background: gradient(theme.colors.intent.none.base, lighten(theme.colors.intent.none.base, 0.6)),
                        boxShadow: insetShadow(alpha(theme.colors.palette.gray.dark, 0.9), alpha(theme.colors.palette.gray.dark, 0.8)),
                        '&[aria-expanded="true"]': {
                            background: "none",
                            backgroundColor: theme.colors.intent.none.base,
                            boxShadow: insetShadow(alpha(palette.gray.base, 1), alpha(palette.gray.base, 0.9))
                        }
                    },
                    active && {
                        background: "none",
                        backgroundColor: theme.colors.intent.none.base,
                        boxShadow: insetShadow(alpha(theme.colors.palette.gray.dark, 1), alpha(theme.colors.palette.gray.dark, 0.9))
                    }
                ]);
            return css([
                {
                    backgroundColor: "white",
                    color: getTextColor(theme.colors.intent.none.lightest, theme),
                    background: gradient(theme.colors.intent.none.lightest, "white"),
                    boxShadow: insetShadow(alpha(theme.colors.palette.gray.dark, 0.2), alpha(theme.colors.palette.gray.dark, 0.1)),
                    '&[aria-expanded="true"]': {
                        background: "none",
                        backgroundColor: theme.colors.intent.none.lightest,
                        boxShadow: insetShadow(alpha(palette.gray.dark, 0.3), alpha(palette.gray.dark, 0.15))
                    }
                },
                active && {
                    background: "none",
                    backgroundColor: theme.colors.intent.none.lightest,
                    boxShadow: insetShadow(alpha(palette.gray.dark, 0.3), alpha(palette.gray.dark, 0.15))
                }
            ]);
        }
        case "primary":
            return css([
                {
                    backgroundColor: theme.colors.intent.primary.base,
                    color: getTextColor(theme.colors.intent.primary.base, theme),
                    backgroundImage: gradient(theme.colors.intent.primary.base, color(theme.colors.intent.primary.base)
                        .lighten(0.4)
                        .toString()),
                    boxShadow: dark
                        ? insetShadow(theme.colors.palette.gray.dark, theme.colors.palette.gray.dark)
                        : insetShadow(alpha(palette.gray.dark, 0.2), alpha(palette.gray.dark, 0.2)),
                    '&[aria-expanded="true"]': {
                        boxShadow: activeShadow,
                        backgroundImage: gradient(theme.colors.intent.primary.dark, theme.colors.intent.primary.base)
                    }
                },
                active && {
                    boxShadow: activeShadow,
                    backgroundImage: gradient(theme.colors.intent.primary.dark, theme.colors.intent.primary.base)
                }
            ]);
        case "success":
            return css([
                {
                    backgroundColor: theme.colors.intent.success.base,
                    color: getTextColor(theme.colors.intent.success.base, theme),
                    backgroundImage: gradient(theme.colors.intent.success.base, color(theme.colors.intent.success.base)
                        .lighten(0.5)
                        .toString()),
                    boxShadow: dark
                        ? insetShadow(theme.colors.palette.gray.dark, theme.colors.palette.gray.dark)
                        : insetShadow(alpha(palette.gray.dark, 0.2), alpha(palette.gray.dark, 0.2)),
                    '&[aria-expanded="true"]': {
                        boxShadow: activeShadow,
                        backgroundImage: gradient(color(theme.colors.intent.success.base)
                            .darken(0.2)
                            .toString(), theme.colors.intent.success.base)
                    }
                },
                active && {
                    boxShadow: activeShadow,
                    backgroundImage: gradient(color(theme.colors.intent.success.base)
                        .darken(0.2)
                        .toString(), theme.colors.intent.success.base)
                }
            ]);
        case "danger":
            return css([
                {
                    backgroundColor: theme.colors.intent.danger.base,
                    color: getTextColor(theme.colors.intent.danger.base, theme),
                    backgroundImage: gradient(theme.colors.intent.danger.base, color(theme.colors.intent.danger.base)
                        .lighten(0.3)
                        .toString()),
                    boxShadow: dark
                        ? insetShadow(theme.colors.palette.gray.dark, theme.colors.palette.gray.dark)
                        : insetShadow(alpha(palette.gray.dark, 0.2), alpha(palette.gray.dark, 0.2)),
                    '&[aria-expanded="true"]': {
                        boxShadow: activeShadow,
                        backgroundImage: gradient(color(theme.colors.intent.danger.base)
                            .darken(0.2)
                            .toString(), theme.colors.intent.danger.base)
                    }
                },
                active && {
                    boxShadow: activeShadow,
                    backgroundImage: gradient(color(theme.colors.intent.danger.base)
                        .darken(0.2)
                        .toString(), theme.colors.intent.danger.base)
                }
            ]);
        case "warning":
            return css([
                {
                    backgroundColor: theme.colors.intent.warning.base,
                    color: getTextColor(theme.colors.intent.warning.base, theme),
                    backgroundImage: gradient(theme.colors.intent.warning.base, color(theme.colors.intent.warning.base)
                        .lighten(0.4)
                        .toString()),
                    boxShadow: dark
                        ? insetShadow(theme.colors.palette.gray.dark, theme.colors.palette.gray.dark)
                        : insetShadow(alpha(palette.gray.dark, 0.2), alpha(palette.gray.dark, 0.2)),
                    '&[aria-expanded="true"]': {
                        boxShadow: activeShadow,
                        backgroundImage: gradient(color(theme.colors.intent.warning.base)
                            .darken(0.2)
                            .toString(), theme.colors.intent.warning.base)
                    }
                },
                active && {
                    boxShadow: activeShadow,
                    backgroundImage: gradient(color(theme.colors.intent.warning.base)
                        .darken(0.2)
                        .toString(), theme.colors.intent.warning.base)
                }
            ]);
    }
};
// variations to the ghost buttons based on intent
const ghostIntents = (theme, intent, hover, active) => {
    const none = intent === "none";
    const dark = theme.colors.mode === "dark";
    let base = none
        ? theme.colors.text.default
        : theme.colors.intent[intent].base;
    if (dark && !none) {
        base = theme.colors.intent[intent].light;
    }
    return css([
        {
            color: none ? theme.colors.text.muted : base,
            opacity: 1,
            background: "transparent",
            boxShadow: "none",
            transition: "box-shadow 0.07s cubic-bezier(0.35,0,0.25,1), background 0.07s cubic-bezier(0.35,0,0.25,1), opacity 0.35s cubic-bezier(0.35,0,0.25,1)",
            '&[aria-expanded="true"]': {
                background: alpha(base, 0.15),
                boxShadow: "none",
                opacity: 1
            }
        },
        hover && {
            background: alpha(base, 0.07)
        },
        active && {
            background: alpha(base, 0.15),
            boxShadow: "none",
            opacity: 1
        }
    ]);
};
// variations to the outline buttons based on intent
const outlineIntents = (theme, intent, hover, active) => {
    const none = intent === "none";
    const dark = theme.colors.mode === "dark";
    let base = none
        ? theme.colors.text.default
        : theme.colors.intent[intent].base;
    if (dark && !none) {
        base = theme.colors.intent[intent].light;
    }
    return css([
        {
            color: base,
            borderColor: alpha(base, 0.2),
            boxShadow: "none",
            transition: "box-shadow 0.07s cubic-bezier(0.35,0,0.25,1), background 0.07s cubic-bezier(0.35,0,0.25,1)",
            '&[aria-expanded="true"]': {
                background: alpha(base, 0.15)
            }
        },
        hover && {
            background: alpha(base, 0.05)
        },
        active && {
            background: alpha(base, 0.15)
        }
    ]);
};
const variants = {
    default: css({}),
    ghost: css({
        background: "transparent",
        boxShadow: "none"
    }),
    outline: css({
        border: `1px solid`,
        background: "none"
    })
};
const iconSpaceForSize = (theme) => ({
    xs: `calc(${theme.iconSizes.xs} + 0.6rem)`,
    sm: `calc(${theme.iconSizes.sm} + 0.6rem)`,
    md: `calc(${theme.iconSizes.md} + 0.6rem)`,
    lg: `calc(${theme.iconSizes.lg} + 0.6rem)`,
    xl: `calc(${theme.iconSizes.xl} + 0.6rem)`
});
/**
 * Your standard Button element
 * 新版本Button 不能传入ref; 若需要控制捕捉,ref传递的，请用ButtonRefComp;
 */
export const Button = ({ size = "md", block, variant = "default", intent = "none", className = "", disabled = false, loading = false, component: Component = "div", iconBefore, iconAfter, children, pressDelay = 0, pressExpandPx, onPress, noBind = false, terminateOnScroll = false, ...other }) => {
    const theme = useTheme();
    const isLink = other.to || other.href;
    const { bind, hover, active } = usePressable({
        onPress,
        disabled,
        behavior: isLink ? "link" : "button"
    });
    // how necessary is this?
    const intentStyle = React.useMemo(() => getIntent(variant, intent, theme, hover, active), [variant, intent, theme, hover, active]);
    if (other.type === "submit" && Component === "div") {
        Component = "button";
    }
    return (_jsxs(Component, { className: cx("Button", "Touchable", className, {
            "Touchable--hover": hover,
            "Touchable--active": active
        }), role: isLink ? "link" : "button", tabIndex: 0, css: [
            buttonReset,
            {
                width: block ? "100%" : undefined,
                fontWeight: 500,
                position: "relative",
                fontFamily: theme.fonts.base,
                borderRadius: theme.radii.sm,
                fontSize: getFontSize(size, theme),
                padding: getPadding(size, 'default'),
                [theme.mediaQueries.sm]: {
                    padding: getPadding(size, 'sm'),
                    whiteSpace: 'unset',
                },
                [theme.mediaQueries.lg]: {
                    padding: getPadding(size, 'lg'),
                },
                height: getHeight(size),
                display: getDisplay(block),
                alignItems: "center",
                justifyContent: "center",
                ":focus": {
                    outline: theme.outline
                },
                ":focus:not([data-focus-visible-added])": {
                    outline: other.autoFocus ? "theme.outline" : "none"
                }
            },
            variants[variant],
            intentStyle,
            disabled && {
                opacity: 0.6,
                pointerEvents: "none"
            },
            loading && {
                "& > :not(.Spinner)": {
                    opacity: 0,
                    transition: "opacity 0.1s ease"
                }
            },
            (iconBefore || (block && iconAfter)) && {
                paddingLeft: "0.65rem"
            },
            (iconAfter || (block && iconBefore)) && {
                paddingRight: "0.65rem"
            },
            {
                "@media print": {
                    boxShadow: 'unset',
                    transition: 'unset',
                },
            }
        ], ...(noBind ? other : safeBind(bind, other)), children: [loading && (_jsx("div", { className: "Spinner", css: {
                    position: "absolute",
                    left: "50%",
                    top: "50%",
                    transform: "translate(-50%, -50%)"
                }, children: _jsx(Spinner, { delay: 1, doing: loading, size: size }) })), iconBefore && (_jsx(IconWrapper, { css: { marginRight: theme.spaces.sm }, size: size, color: "currentColor", "aria-hidden": true, children: iconBefore })), typeof children === "string" ? (_jsx("span", { css: {
                    flex: 1,
                    // kinda lame hack to center our text in block
                    // mode with icons before and after
                    marginRight: block && iconBefore && !iconAfter
                        ? iconSpaceForSize(theme)[size]
                        : 0,
                    marginLeft: block && iconAfter && !iconBefore
                        ? iconSpaceForSize(theme)[size]
                        : 0
                }, "aria-hidden": loading, children: children })) : (children), iconAfter && (_jsx(IconWrapper, { color: "currentColor", css: { marginLeft: theme.spaces.sm }, size: size, children: iconAfter }))] }));
};
/**
 * 旧版本 可支持传入 ref 控制参数的。
 * ButtonRefComp 比起 Button多个 ref需要绑定。 其余代码完全一致
 */
export const ButtonRefComp = React.forwardRef(({ size = "md", block, variant = "default", intent = "none", className = "", disabled = false, loading = false, component: Component = "div", iconBefore, iconAfter, children, pressDelay = 0, pressExpandPx, onPress, noBind = false, terminateOnScroll = false, ...other }, ref) => {
    const theme = useTheme();
    const isLink = other.to || other.href;
    const { bind, hover, active } = usePressable({
        onPress,
        disabled,
        behavior: isLink ? "link" : "button"
    });
    // how necessary is this?
    const intentStyle = React.useMemo(() => getIntent(variant, intent, theme, hover, active), [variant, intent, theme, hover, active]);
    //最终追溯到 DOM 的tag; 基础tag: div button 这些原生的html, 都可以支持ref参考传递。
    if (other.type === "submit" && Component === "div") {
        Component = "button";
    }
    return (_jsxs(Component, { className: cx("Button", "Touchable", className, {
            "Touchable--hover": hover,
            "Touchable--active": active
        }), role: isLink ? "link" : "button", tabIndex: 0, css: [
            buttonReset,
            {
                width: block ? "100%" : undefined,
                fontWeight: 500,
                position: "relative",
                fontFamily: theme.fonts.base,
                borderRadius: theme.radii.sm,
                fontSize: getFontSize(size, theme),
                padding: getPadding(size, 'default'),
                [theme.mediaQueries.sm]: {
                    padding: getPadding(size, 'sm'),
                    whiteSpace: 'unset',
                },
                [theme.mediaQueries.lg]: {
                    padding: getPadding(size, 'lg'),
                },
                height: getHeight(size),
                display: getDisplay(block),
                alignItems: "center",
                justifyContent: "center",
                ":focus": {
                    outline: theme.outline
                },
                ":focus:not([data-focus-visible-added])": {
                    outline: other.autoFocus ? "theme.outline" : "none"
                }
            },
            variants[variant],
            intentStyle,
            disabled && {
                opacity: 0.6,
                pointerEvents: "none"
            },
            loading && {
                "& > :not(.Spinner)": {
                    opacity: 0,
                    transition: "opacity 0.1s ease"
                }
            },
            (iconBefore || (block && iconAfter)) && {
                paddingLeft: "0.65rem"
            },
            (iconAfter || (block && iconBefore)) && {
                paddingRight: "0.65rem"
            },
            {
                "@media print": {
                    boxShadow: 'unset',
                    transition: 'unset',
                },
            }
        ], ...(noBind ? safeBind({ ref }, other) : safeBind(bind, { ref }, other)), children: [loading && (_jsx("div", { className: "Spinner", css: {
                    position: "absolute",
                    left: "50%",
                    top: "50%",
                    transform: "translate(-50%, -50%)"
                }, children: _jsx(Spinner, { delay: 1, doing: loading, size: size }) })), iconBefore && (_jsx(IconWrapper, { css: { marginRight: theme.spaces.sm }, size: size, color: "currentColor", "aria-hidden": true, children: iconBefore })), typeof children === "string" ? (_jsx("span", { css: {
                    flex: 1,
                    // kinda lame hack to center our text in block
                    // mode with icons before and after
                    marginRight: block && iconBefore && !iconAfter
                        ? iconSpaceForSize(theme)[size]
                        : 0,
                    marginLeft: block && iconAfter && !iconBefore
                        ? iconSpaceForSize(theme)[size]
                        : 0
                }, "aria-hidden": loading, children: children })) : (children), iconAfter && (_jsx(IconWrapper, { color: "currentColor", css: { marginLeft: theme.spaces.sm }, size: size, children: iconAfter }))] }));
});
Button.displayName = "Button";
Button.propTypes = {
    variant: PropTypes.oneOf(["default", "ghost", "outline"]),
    size: PropTypes.oneOf(["xs", "sm", "md", "lg", "xl"]),
    block: PropTypes.bool,
    intent: PropTypes.oneOf([
        "none",
        "primary",
        "success",
        "warning",
        "danger"
    ]),
    iconBefore: PropTypes.node,
    iconAfter: PropTypes.node
};
function getIntent(variant, intent, theme, hover, active) {
    switch (variant) {
        case "default":
            return getIntentStyles(theme, intent, hover, active);
        case "ghost":
            return ghostIntents(theme, intent, hover, active);
        case "outline":
            return outlineIntents(theme, intent, hover, active);
    }
}
//# sourceMappingURL=Button.js.map