import { jsx as _jsx, Fragment as _Fragment, jsxs as _jsxs } from "@emotion/react/jsx-runtime";
import { forwardRef, useRef, useState } from "react";
import { autoUpdate, size, useId, useDismiss, useFloating, useInteractions, useListNavigation, useRole, FloatingFocusManager, FloatingPortal } from "@floating-ui/react";
import { BlurTriggerRefInput } from "./Form";
const Item = forwardRef(({ children, active, index, ...rest }, ref) => {
    const id = useId();
    return (_jsx("div", { ref: ref, role: "option", id: id, "aria-selected": active, ...rest, style: {
            background: active ? "lightblue" : index % 2 === 0 ? "none" : "lavender",
            padding: 4,
            cursor: "default",
            ...rest.style
        }, children: children }));
});
/*输入组合框： 原装例子 的 AutoComplete();
只能是文本内容 datalist；
旧版本用了Highlighter高亮。
* */
export function ComboBoxDatalist({ autoComplete, autoFocus, inputSize = "md", fullWidth = true, datalist = [], style, placeholder, value, onListChange, ...other }) {
    const [open, setOpen] = useState(false);
    const [inputValue, setInputValue] = useState(value);
    const [activeIndex, setActiveIndex] = useState(null);
    const listRef = useRef([]);
    const { x, y, reference, floating, strategy, context } = useFloating({
        whileElementsMounted: autoUpdate,
        open,
        onOpenChange: setOpen,
        middleware: [
            size({
                apply({ rects, availableHeight, elements }) {
                    Object.assign(elements.floating.style, {
                        width: `${rects.reference.width}px`,
                        maxHeight: `${availableHeight}px`
                    });
                },
                padding: 10
            })
        ]
    });
    // useLayoutEffect(() => {
    //     // IMPORTANT: When the floating element first opens, this effect runs when
    //     // the styles have **not yet** been applied to the element. A rAF ensures
    //     // we wait until the position is ready, and also runs before paint.
    //     // https://floating-ui.com/docs/react-dom#effects
    //     requestAnimationFrame(() => {
    //         if (activeIndex != null) {
    //             listRef.current[activeIndex]?.scrollIntoView({ block: "nearest" });
    //         }
    //     });
    // }, [activeIndex]);
    const { getReferenceProps, getFloatingProps, getItemProps } = useInteractions([
        useRole(context, { role: "listbox" }),
        useDismiss(context),
        useListNavigation(context, {
            listRef,
            activeIndex,
            onNavigate: setActiveIndex,
            virtual: true,
            loop: true
        })
    ]);
    function onChange(event) {
        const value = event.target.value;
        setInputValue(value);
        if (value) {
            setOpen(true);
            setActiveIndex(0);
        }
        else {
            setOpen(false);
        }
    }
    const items = inputValue ? datalist.filter((item) => item.toLowerCase().includes(inputValue.toLowerCase()))
        :
            datalist; //全部可看
    return (_jsxs(_Fragment, { children: [_jsx(BlurTriggerRefInput, { onSave: function (text) {
                    onListChange(text);
                }, ...getReferenceProps({
                    ref: reference,
                    onChange,
                    value: inputValue,
                    placeholder: placeholder,
                    "aria-autocomplete": "list",
                    onKeyDown(event) {
                        if (event.key === "Enter" &&
                            activeIndex != null &&
                            items[activeIndex]) {
                            setInputValue(items[activeIndex]);
                            setActiveIndex(null);
                            setOpen(false);
                        }
                    },
                    onPointerDown(event) {
                        setOpen(true);
                    },
                    style: {
                        ...style,
                    }
                }), ...other }), _jsx(FloatingPortal, { children: open && (_jsx(FloatingFocusManager, { context: context, initialFocus: -1, visuallyHiddenDismiss: true, children: _jsx("div", { ...getFloatingProps({
                            ref: floating,
                            style: {
                                position: strategy,
                                left: x ?? 0,
                                top: y ?? 0,
                                background: "#eee",
                                color: "black",
                                overflowY: "auto"
                            }
                        }), children: items.map((item, index) => (_jsx(Item, { index: index, ...getItemProps({
                                key: item,
                                ref(node) {
                                    listRef.current[index] = node;
                                },
                                onClick() {
                                    setInputValue(item);
                                    setOpen(false);
                                    //refs.reference.current?.focus();
                                },
                                style: {
                                    listStyleType: 'none',
                                    padding: 0,
                                    margin: 0,
                                },
                            }), active: activeIndex === index, children: item }))) }) })) })] }));
}
//# sourceMappingURL=ComboBoxDatalist.js.map