import { jsx as _jsx, jsxs as _jsxs } from "@emotion/react/jsx-runtime";
/** @jsxImportSource @emotion/react */
import * as React from "react";
import { Text } from "./Text";
import PropTypes from "prop-types";
//旧版本依赖Sheet提供了 import { RequestCloseContext } from "./Sheet";
import { useTheme } from "./Theme/Providers";
import { noOp } from "./misc/noop";
// import { useTouchable,  } from "./Hooks/touchable-hook";
import cx from "classnames";
import { safeBind } from "./Hooks/compose-bind";
import { forwardRef } from "react";
const KeyCodes = {
    ArrowUp: 38,
    ArrowDown: 40,
    Home: 36,
    End: 35
};
const MenuListContext = React.createContext({
    focus: false,
    onKeyDown: () => { }
});
export const MenuList = ({ children, focusableChildren = [], ...other }) => {
    const theme = useTheme();
    const disabled = new Map();
    const [focusIndex, setFocusIndex] = React.useState(null);
    const kids = React.Children.toArray(children);
    const focusComponents = [MenuItem, ...focusableChildren];
    const focusableItems = kids.filter(kid => React.isValidElement(kid) &&
        focusComponents.indexOf(kid.type) > -1);
    const lastIndex = focusableItems.length - 1;
    const firstIndex = 0;
    return (_jsx("div", { role: "menu", onKeyDown: (e) => {
            if (e.keyCode === KeyCodes.ArrowDown ||
                (e.keyCode === KeyCodes.ArrowUp && focusIndex === null)) {
                e.preventDefault();
                setFocusIndex(0);
            }
            else if (e.keyCode === 13) {
                //菜单列表 回车 有何意义
                e.stopPropagation();
                e.preventDefault();
            }
        }, css: {
            minWidth: "200px",
            display: "block",
            padding: `${theme.spaces.sm} 0`
        }, ...other, children: kids.map(kid => {
            if (!React.isValidElement(kid)) {
                return null;
            }
            const i = focusableItems.indexOf(kid);
            if (i < 0) {
                return kid;
            }
            disabled.set(i, kid.props.disabled);
            function focusDown(current) {
                const next = current + 1 > lastIndex ? firstIndex : current + 1;
                if (disabled.get(next)) {
                    focusDown(next);
                }
                else {
                    setFocusIndex(next);
                }
            }
            function focusUp(current) {
                const next = current - 1 < firstIndex ? lastIndex : current - 1;
                if (disabled.get(next)) {
                    focusUp(next);
                }
                else {
                    setFocusIndex(next);
                }
            }
            const index = focusIndex || 0;
            return (_jsx(MenuListContext.Provider, { value: {
                    // !第一个菜单项目 默认就是focus= true;
                    focus: i === focusIndex,
                    onKeyDown: (e) => {
                        if (e.keyCode === KeyCodes.ArrowDown) {
                            e.preventDefault();
                            focusDown(index);
                        }
                        else if (e.keyCode === KeyCodes.ArrowUp) {
                            e.preventDefault();
                            focusUp(index);
                        }
                        else if (e.keyCode === KeyCodes.Home) {
                            e.preventDefault();
                            setFocusIndex(firstIndex);
                        }
                        else if (e.keyCode === KeyCodes.End) {
                            e.preventDefault();
                            setFocusIndex(lastIndex);
                        }
                        else if (e.keyCode === 13) {
                            //菜单列表 回车 有何意义
                            e.stopPropagation();
                            e.preventDefault();
                        }
                    }
                }, children: kid }, i));
        }) }));
};
MenuList.propTypes = {
//focusableChildren: PropTypes.arrayOf(PropTypes.element)
};
export function MenuDivider(props) {
    const theme = useTheme();
    return (_jsx("div", { css: {
            height: 0,
            margin: `${theme.spaces.sm} 0`,
            overflow: "hidden",
            borderTop: "1px solid",
            borderColor: theme.colors.border.muted
        }, ...props }));
}
export const MenuLabel = props => {
    const theme = useTheme();
    return (_jsx(Text, { variant: "uppercase", css: {
            padding: `${theme.spaces.xs} ${theme.spaces.md}`
        }, ...props }));
};
MenuLabel.propTypes = {
    children: PropTypes.node
};
/**将来准备恢复成 MenuItem ？参数不兼容onClick onPress;
 * 非touch事件版,onClick()时间;  DdMenu适用的。
 * 因为从外部注入的方式修正padding 需要像这样的全部都要选择上，或者 但！important！的也太武断
 divStyle={{
        padding: 0,
        [theme.mediaQueries.sm]: {
            padding: 0,
        },
        [theme.mediaQueries.md]: {
            padding: 0,
        },
        [theme.mediaQueries.lg]: {
            padding: 0,
        },
        [theme.mediaQueries.xl]: {
            padding: 0,
        },
    }}
 考虑加一个参数：tight 表示是否紧凑的菜单按钮。
 * */
export const MenuItem = forwardRef(({ contentBefore, contentAfter, className = "", component: Component = "div", onPress = noOp, role = "menuitem", children, disabled, 
// onClick,
tight, ...other }, ref) => {
    const theme = useTheme();
    const dark = theme.colors.mode === "dark";
    //const touchScr = useMedia(`(any-pointer: coarse)`);
    //手机和大屏幕的内容维持一致：菜单区域大小高度应该差不多。
    return (_jsxs(Component, { className: cx("MenuItem", "Touchable", className), css: [
            {
                cursor: "pointer",
                opacity: disabled ? 0.3 : 1,
                display: "flex",
                lineHeight: theme.lineHeights.menus,
                textDecoration: "none",
                transition: "background-color 0.1s ease",
                WebkitTapHighlightColor: "transparent",
                color: theme.colors.text.default,
                alignItems: "center",
                ":focus": {
                    outline: theme.outline
                },
                ":focus:not([data-focus-visible-added])": {
                    outline: "none"
                },
                ":hover": {
                    background: theme.colors.background.tint1
                },
                padding: 0,
                "@media print": {
                    transition: 'unset',
                },
            },
            !tight && ({
                padding: `${theme.spaces.xs} ${theme.spaces.sm}`,
                [theme.mediaQueries.sm]: {
                    padding: `${theme.spaces.none} ${theme.spaces.none}`
                },
                [theme.mediaQueries.md]: {
                    padding: `${theme.spaces.xs} calc(${theme.spaces.xs} + 0.35rem)`,
                },
                [theme.mediaQueries.lg]: {
                    padding: `${theme.spaces.xs} ${theme.spaces.md}`
                },
                [theme.mediaQueries.xl]: {
                    padding: `${theme.spaces.sm} ${theme.spaces.lg}`
                },
            }),
        ], role: role, tabIndex: disabled ? -1 : 0, "data-trigger-close": true, ...safeBind({
            ref: ref,
            onClick: (e) => {
                // console.log("弹出3菜Menu2ImmT !点了");
                !disabled && onPress(e);
            },
            // onPointerUp: (e: React.PointerEvent ) => {  基本点击触发
            //  加上能使得DdMenu在位于Panable可随意拉拽的组件底下，能够触发"lostpointercapture"的useDrag(state)消息event.type
            //     e.stopPropagation();  }
        }, other), children: [contentBefore, typeof children === "string" ? (_jsx(Text, { wrap: true, css: {
                    paddingLeft: contentBefore ? theme.spaces.sm : 0,
                    paddingRight: contentAfter ? theme.spaces.sm : 0,
                    flex: 1,
                    color: "inherit",
                    lineHeight: theme.lineHeights.menus,
                }, children: children })) : (children), typeof contentAfter === "string" ? (_jsx(Text, { muted: true, children: contentAfter })) : (contentAfter)] }));
});
//# sourceMappingURL=Menu.js.map