import { jsx as _jsx } from "@emotion/react/jsx-runtime";
/** @jsxImportSource @emotion/react */
import { css } from "@emotion/react";
import * as React from "react";
import PropTypes from "prop-types";
import { useTheme } from "./Theme/Providers";
const getStyles = (theme) => ({
    fixed: css({
        position: "fixed",
        top: 0,
        left: 0,
        width: "100%",
        zIndex: theme.zIndices.fixed
    }),
    static: css({
        position: "static",
        boxShadow: "none"
    }),
    absolute: css({
        position: "absolute",
        top: 0,
        left: 0,
        width: "100%"
    }),
    sticky: css({
        position: "sticky",
        top: 0,
        zIndex: theme.zIndices.sticky
    })
});
/**
 * A Navbar is used to provide app based navigation. Typically
 * used in combination with a Toolbar.
 */
export const Navbar = ({ position = "fixed", children, ...other }) => {
    const theme = useTheme();
    const styles = React.useMemo(() => getStyles(theme), [theme]);
    return (_jsx("nav", { css: [
            {
                background: theme.colors.background.default,
                zIndex: position === "fixed" ? theme.zIndices.fixed : undefined,
                boxShadow: theme.shadows.sm,
                "@media print": {
                    boxShadow: 'unset',
                },
            },
            styles[position]
        ], ...other, children: children }));
};
Navbar.propTypes = {
    //position: PropTypes.oneOf(["fixed", "static"] as Positions[]),  ? sticky
    children: PropTypes.node
};
//# sourceMappingURL=Navbar.js.map