/** @jsxImportSource @emotion/react */
import React, { ReactNode } from "react";
export declare const sleep: (milliseconds: number) => Promise<unknown>;
interface PrintProps {
    children: ReactNode;
    onOpenPrintDialog?: () => void | Promise<void>;
    onClosePrintDialog?: () => void | Promise<void>;
    printWidth?: number;
    marginTop?: number;
    marginLeft?: number;
    canURLprn?: boolean;
    waitime?: number;
}
/**为整个URL触发打印语境。
 * 为了完全动态内容正常打印而生的： 数据必须加载进来，但是不能动态人工修改和输入没保存后端就打印的。
 * 支持打印转发器用。不需要打印对话框也能够正常对完全动态网页 打印，不会在尾部有空白页或缺少应该打印的页。
 * 设置 canURLprn=true & PRINTpdf=1 后才启动。否则不启用（走的保留旧版本按钮驱动的）
 * @test：没有启动的 简易版原始记录的测试：输出68页 启动后自动调整显示区宽度的 输出69页，针对许多依据测量div宽度来动态布局LineColumn的情况。fl14-1；
 * URL参数： ？ & PRINTwidth=718  约束打印纸张的宽度；
 * 工具条肯定不会打印的，隐藏预定义的工具条： #floormenu 元素ID='floormenu'；
 * 手机上chrome打印可能有些问题；
 * */
export declare const UrlPrint: React.ForwardRefExoticComponent<Pick<PrintProps & React.HTMLProps<HTMLDivElement>, "children" | "value" | "cite" | "data" | "form" | "label" | "slot" | "span" | "style" | "summary" | "title" | "pattern" | "hidden" | "wrap" | "muted" | "defaultChecked" | "defaultValue" | "suppressContentEditableWarning" | "suppressHydrationWarning" | "accessKey" | "list" | "className" | "contentEditable" | "contextMenu" | "dir" | "draggable" | "id" | "lang" | "nonce" | "placeholder" | "spellCheck" | "tabIndex" | "translate" | "radioGroup" | "role" | "about" | "datatype" | "inlist" | "prefix" | "property" | "resource" | "typeof" | "vocab" | "autoCapitalize" | "autoCorrect" | "autoSave" | "color" | "itemProp" | "itemScope" | "itemType" | "itemID" | "itemRef" | "results" | "security" | "unselectable" | "inputMode" | "is" | "aria-activedescendant" | "aria-atomic" | "aria-autocomplete" | "aria-busy" | "aria-checked" | "aria-colcount" | "aria-colindex" | "aria-colspan" | "aria-controls" | "aria-current" | "aria-describedby" | "aria-details" | "aria-disabled" | "aria-dropeffect" | "aria-errormessage" | "aria-expanded" | "aria-flowto" | "aria-grabbed" | "aria-haspopup" | "aria-hidden" | "aria-invalid" | "aria-keyshortcuts" | "aria-label" | "aria-labelledby" | "aria-level" | "aria-live" | "aria-modal" | "aria-multiline" | "aria-multiselectable" | "aria-orientation" | "aria-owns" | "aria-placeholder" | "aria-posinset" | "aria-pressed" | "aria-readonly" | "aria-relevant" | "aria-required" | "aria-roledescription" | "aria-rowcount" | "aria-rowindex" | "aria-rowspan" | "aria-selected" | "aria-setsize" | "aria-sort" | "aria-valuemax" | "aria-valuemin" | "aria-valuenow" | "aria-valuetext" | "dangerouslySetInnerHTML" | "onCopy" | "onCopyCapture" | "onCut" | "onCutCapture" | "onPaste" | "onPasteCapture" | "onCompositionEnd" | "onCompositionEndCapture" | "onCompositionStart" | "onCompositionStartCapture" | "onCompositionUpdate" | "onCompositionUpdateCapture" | "onFocus" | "onFocusCapture" | "onBlur" | "onBlurCapture" | "onChange" | "onChangeCapture" | "onBeforeInput" | "onBeforeInputCapture" | "onInput" | "onInputCapture" | "onReset" | "onResetCapture" | "onSubmit" | "onSubmitCapture" | "onInvalid" | "onInvalidCapture" | "onLoad" | "onLoadCapture" | "onError" | "onErrorCapture" | "onKeyDown" | "onKeyDownCapture" | "onKeyPress" | "onKeyPressCapture" | "onKeyUp" | "onKeyUpCapture" | "onAbort" | "onAbortCapture" | "onCanPlay" | "onCanPlayCapture" | "onCanPlayThrough" | "onCanPlayThroughCapture" | "onDurationChange" | "onDurationChangeCapture" | "onEmptied" | "onEmptiedCapture" | "onEncrypted" | "onEncryptedCapture" | "onEnded" | "onEndedCapture" | "onLoadedData" | "onLoadedDataCapture" | "onLoadedMetadata" | "onLoadedMetadataCapture" | "onLoadStart" | "onLoadStartCapture" | "onPause" | "onPauseCapture" | "onPlay" | "onPlayCapture" | "onPlaying" | "onPlayingCapture" | "onProgress" | "onProgressCapture" | "onRateChange" | "onRateChangeCapture" | "onResize" | "onResizeCapture" | "onSeeked" | "onSeekedCapture" | "onSeeking" | "onSeekingCapture" | "onStalled" | "onStalledCapture" | "onSuspend" | "onSuspendCapture" | "onTimeUpdate" | "onTimeUpdateCapture" | "onVolumeChange" | "onVolumeChangeCapture" | "onWaiting" | "onWaitingCapture" | "onAuxClick" | "onAuxClickCapture" | "onClick" | "onClickCapture" | "onContextMenu" | "onContextMenuCapture" | "onDoubleClick" | "onDoubleClickCapture" | "onDrag" | "onDragCapture" | "onDragEnd" | "onDragEndCapture" | "onDragEnter" | "onDragEnterCapture" | "onDragExit" | "onDragExitCapture" | "onDragLeave" | "onDragLeaveCapture" | "onDragOver" | "onDragOverCapture" | "onDragStart" | "onDragStartCapture" | "onDrop" | "onDropCapture" | "onMouseDown" | "onMouseDownCapture" | "onMouseEnter" | "onMouseLeave" | "onMouseMove" | "onMouseMoveCapture" | "onMouseOut" | "onMouseOutCapture" | "onMouseOver" | "onMouseOverCapture" | "onMouseUp" | "onMouseUpCapture" | "onSelect" | "onSelectCapture" | "onTouchCancel" | "onTouchCancelCapture" | "onTouchEnd" | "onTouchEndCapture" | "onTouchMove" | "onTouchMoveCapture" | "onTouchStart" | "onTouchStartCapture" | "onPointerDown" | "onPointerDownCapture" | "onPointerMove" | "onPointerMoveCapture" | "onPointerUp" | "onPointerUpCapture" | "onPointerCancel" | "onPointerCancelCapture" | "onPointerEnter" | "onPointerEnterCapture" | "onPointerLeave" | "onPointerLeaveCapture" | "onPointerOver" | "onPointerOverCapture" | "onPointerOut" | "onPointerOutCapture" | "onGotPointerCapture" | "onGotPointerCaptureCapture" | "onLostPointerCapture" | "onLostPointerCaptureCapture" | "onScroll" | "onScrollCapture" | "onWheel" | "onWheelCapture" | "onAnimationStart" | "onAnimationStartCapture" | "onAnimationEnd" | "onAnimationEndCapture" | "onAnimationIteration" | "onAnimationIterationCapture" | "onTransitionEnd" | "onTransitionEndCapture" | "htmlFor" | "content" | "height" | "marginLeft" | "marginTop" | "width" | "key" | "css" | "size" | "start" | "disabled" | "default" | "autoFocus" | "formAction" | "formEncType" | "formMethod" | "formNoValidate" | "formTarget" | "name" | "type" | "step" | "href" | "max" | "media" | "method" | "min" | "target" | "crossOrigin" | "classID" | "as" | "download" | "alt" | "coords" | "autoPlay" | "controls" | "dateTime" | "open" | "acceptCharset" | "action" | "autoComplete" | "encType" | "manifest" | "allowFullScreen" | "allowTransparency" | "frameBorder" | "accept" | "capture" | "checked" | "challenge" | "keyType" | "keyParams" | "charSet" | "httpEquiv" | "high" | "low" | "reversed" | "selected" | "async" | "multiple" | "cellPadding" | "cellSpacing" | "colSpan" | "headers" | "rowSpan" | "scope" | "cols" | "maxLength" | "minLength" | "kind" | "src" | "srcLang" | "preload" | "defer" | "hrefLang" | "integrity" | "loop" | "marginHeight" | "marginWidth" | "mediaGroup" | "noValidate" | "optimum" | "playsInline" | "poster" | "readOnly" | "rel" | "required" | "rows" | "sandbox" | "scoped" | "scrolling" | "seamless" | "shape" | "sizes" | "srcDoc" | "srcSet" | "useMap" | "wmode" | "canURLprn" | "printWidth" | "onOpenPrintDialog" | "onClosePrintDialog" | "waitime"> & React.RefAttributes<HTMLDivElement>>;
interface Props {
    node: ReactNode;
    printWidth?: number;
    marginTop?: number;
    marginLeft?: number;
    canURLprn?: boolean;
    waitime?: number;
}
/**仅仅针对单一个组件的打印； 其它元素需要隐藏掉的 ；表格数据编辑: 不能多個表格追加一起做編輯的。
 * 用了<DdMenu icon替代了<Popover>做菜单后会好点：菜单版面紧凑而且易于扩展。
 * 手机上chrome打印可能有些问题；版本？
 * 【毛病】不支持手动输入数据的立刻打印出来，就只能看到URL mount加载来的数据；还会留有许多空白页只有最前面才有内容的。
 * */
export declare function useDomPrinter({ node, canURLprn, waitime, marginTop, marginLeft, printWidth, }: Props): {
    id: string;
    buttonProps: {
        onClick: () => Promise<void>;
        "aria-controls": string;
    };
    render: import("@emotion/react/jsx-runtime").JSX.Element;
};
export {};
/**被动态网页的打印给折腾的：
 重新加载？打印敲定页数之后才会再一次render的。根本没机会计算ref.div最新宽度甚至用beforePrintHandler来setSuitableCls也没有机会提前调整。
 beforePrintHandler --》unmount this?重新加载
 react-to-print只会复制旧的组件页面也不能重新render的是甚至usemedia都无感知到打印。；
 只用css 不用JS做动态内容？；
 <style>
 {
 `:not(.print-comp) { visibility: hidden !important; } body { min-height: 0; height: ${
 usedMarginTop + (dimensions?.height ?? 0)
 }px; } .print-comp, .print-comp * { visibility: visible !important}`
 }
 </style>
 function sleep(time: number | undefined) {
     return new Promise(resolve =>
         setTimeout(resolve,time) )
 }
 async function pressWait() {
     let out = await sleep(9900);
     return out;
 }
* */
