import { jsx as _jsx } from "@emotion/react/jsx-runtime";
import * as React from "react";
import { mergeRefs } from "react-merge-refs";
import { useFloating, autoUpdate, offset, flip, shift, useHover, useFocus, useDismiss, useRole, useInteractions, FloatingPortal } from "@floating-ui/react";
export function useTooltip({ initialOpen = false, placement = "top", open: controlledOpen, onOpenChange: setControlledOpen } = {}) {
    const [uncontrolledOpen, setUncontrolledOpen] = React.useState(initialOpen);
    const open = controlledOpen ?? uncontrolledOpen;
    const setOpen = setControlledOpen ?? setUncontrolledOpen;
    const data = useFloating({
        placement,
        open,
        onOpenChange: setOpen,
        whileElementsMounted: autoUpdate,
        middleware: [offset(5), flip(), shift()]
    });
    const context = data.context;
    const hover = useHover(context, {
        move: false,
        enabled: controlledOpen == null
    });
    const focus = useFocus(context, {
        enabled: controlledOpen == null
    });
    const dismiss = useDismiss(context);
    const role = useRole(context, { role: "tooltip" });
    const interactions = useInteractions([hover, focus, dismiss, role]);
    return React.useMemo(() => ({
        open,
        setOpen,
        ...interactions,
        ...data
    }), [open, setOpen, interactions, data]);
}
const TooltipContext = React.createContext(null);
export const useTooltipState = () => {
    const context = React.useContext(TooltipContext);
    if (context == null) {
        throw new Error("Tooltip components must be wrapped in <Tooltip />");
    }
    return context;
};
//没有箭头了
export function Tooltip({ children, ...options }) {
    // This can accept any props as options, e.g. `placement`,
    // or other positioning options.
    const tooltip = useTooltip(options);
    return (_jsx(TooltipContext.Provider, { value: tooltip, children: children }));
}
export const TooltipTrigger = React.forwardRef(function TooltipTrigger({ children, asChild = false, ...props }, propRef) {
    const state = useTooltipState();
    const childrenRef = children.ref;
    const ref = React.useMemo(() => mergeRefs([state.reference, propRef, childrenRef]), [state.reference, propRef, childrenRef]);
    // `asChild` allows the user to pass any element as the anchor
    if (asChild && React.isValidElement(children)) {
        return React.cloneElement(children, state.getReferenceProps({
            ref,
            ...props,
            ...children.props,
            "data-state": state.open ? "open" : "closed"
        }));
    }
    return (_jsx("button", { ref: ref, "data-state": state.open ? "open" : "closed", ...state.getReferenceProps(props), children: children }));
});
export const TooltipContent = React.forwardRef(function TooltipContent(props, propRef) {
    const state = useTooltipState();
    const ref = React.useMemo(() => mergeRefs([state.floating, propRef]), [
        state.floating,
        propRef
    ]);
    return (_jsx(FloatingPortal, { children: state.open && (_jsx("div", { ref: ref, style: {
                position: state.strategy,
                top: state.y ?? 0,
                left: state.x ?? 0,
                visibility: state.x == null ? "hidden" : "visible",
                width: 'max-content',
                backgroundColor: '#444',
                color: 'white',
                fontSize: '90%',
                padding: '4px 8px',
                borderRadius: '4px',
                ...props.style
            }, ...state.getFloatingProps(props) })) }));
});
//# sourceMappingURL=Tooltip.js.map