import { jsx as _jsx, jsxs as _jsxs } from "@emotion/react/jsx-runtime";
import * as React from "react";
import { useContext } from "react";
import { useMMenuBarOcup, PanelEnlargeCtx } from "./MainMenuBar";
import { useTheme } from "./Theme/Providers";
import { Navbar } from "./Navbar";
import { Toolbar } from "./Toolbar";
import { IconChevronsLeft, IconMoreVertical } from "./Icons";
import { DdMenu, DdMenuItem, } from "./DropdownMenu";
import { MenuItem } from "./Menu";
import { Text } from "./Text";
/**右半边页面框架;   目的是配合<TwoHalfFrame/>共同构建框架页面。
 * 配合使用<TwoHalfFrame/> 这两个组件的共同父辈：可能是静态路由器分割开的，那么需要在路由器的父辈，也就是index.tsx比如这样的:
 * const container = document.getElementById('root');
 * const root = createRoot(container!);
 * root.render(      底下的顶级组件注入 <XxxContext.Provider比如底下这样的：
 const [enlarge, setEnlarge] = React.useState(null);        右半边页面状态是放大吗
 return (
 <PanelEnlargeCtx.Provider  value={{ enlarge, setEnlarge }}>
 {props.children}
 </PanelEnlargeCtx.Provider>
 )}
 注意：可能要设置<TwoHalfRightPanel  key={不同的id} />避免无法更新页面数据。
 * */
export const TwoHalfRightPanel = ({ back, menu, title, children, divStyle, topStyle, enlarge: desiredLarge, noHalfFrame, ...other }) => {
    const theme = useTheme();
    //工具条高度采用与底下的主菜单条的高度一样的设置
    const { carousel, barHeight, bottomCss } = useMMenuBarOcup(false);
    const contextPanel = useContext(PanelEnlargeCtx);
    if (contextPanel === null && !noHalfFrame)
        throw new Error(`没有PanelEnlargeCtx`);
    const { enlarge, setEnlarge } = contextPanel || {};
    React.useEffect(() => {
        if (!carousel) {
            //【奇特】不可以将enlarge也加入到依赖项：否则无法手动调节，像是依赖的死循环了。
            if (enlarge !== desiredLarge && desiredLarge < 4 && desiredLarge >= 0)
                setEnlarge(desiredLarge);
        }
    }, [desiredLarge, carousel, setEnlarge]); //【别管编译警告】enlarge,千万别加上!
    //为了控制手机竖屏 轮播台的居中显示增加顶级div: "RightTop" 才能够适应控制。
    return (_jsxs("div", { className: "RightTop", css: {
            display: 'flex',
            minHeight: carousel ? `calc(100vh - ${barHeight})` : '100%',
            flexDirection: 'column',
            justifyContent: carousel ? 'space-evenly' : 'center',
            ...topStyle,
        }, children: [_jsx(Navbar, { position: "sticky", css: {
                    backgroundColor: "white",
                }, ...other, children: _jsxs(Toolbar, { css: {
                        alignItems: "center",
                        display: "flex",
                        minHeight: `${barHeight}`,
                        //底层Toolbar的设置,要覆盖掉：
                        [theme.mediaQueries.lg]: {
                            minHeight: `${barHeight}`,
                        }
                    }, children: [back, typeof title === "string" ? (_jsx(Text, { css: {
                                flex: 1,
                                textAlign: "center",
                                [theme.mediaQueries.md]: {
                                    textAlign: "left"
                                }
                            }, wrap: false, variant: "h5", gutter: false, children: title }))
                            :
                                (title), _jsxs("div", { css: {
                                display: 'inline-flex'
                            }, children: [menu, !carousel && !noHalfFrame && _jsxs(DdMenu, { label: "\u653E\u5927", tight: true, icon: _jsx(MenuItem, { variant: "ghost", component: "div", children: _jsx(IconChevronsLeft, {}) }), children: [_jsx(DdMenuItem, { label: "\u5DE6\u6EE1\u5C4F", onClick: (e) => { setEnlarge(4); } }), _jsx(DdMenuItem, { label: "1\u6BD42", onClick: (e) => { setEnlarge(1); } }), _jsx(DdMenuItem, { label: "1\u6BD41", onClick: (e) => { setEnlarge(2); } }), _jsx(DdMenuItem, { label: "2\u6BD41", onClick: (e) => { setEnlarge(3); } }), _jsx(DdMenuItem, { label: "\u53F3\u6EE1\u5C4F", onClick: (e) => { setEnlarge(0); } })] })] })] }) }), _jsx("div", { className: "HalfPaneRM", css: [
                    {
                        display: 'flex',
                        alignItems: 'center',
                        justifyContent: 'center',
                        //minHeight: '40vh', 大屏幕下会导致和工具栏分离
                        minHeight: carousel ? '40vh' : 'unset',
                        //paddingBottom:'3rem'
                    },
                    carousel ? bottomCss : enlarge === 4 ? undefined : bottomCss,
                ], children: _jsx("div", { className: "RightContent", css: {
                        width: '100%',
                        background: 'white',
                        ...divStyle,
                    }, children: children }) })] }));
};
/**按钮定制的特殊菜单
 无法使用<VerticalMenu> {传递参数形式} </VerticalMenu>；菜单第一层不能正常地hover点亮和正常关闭。
 只能利用本简化按钮包裹，如下形式：
 <VerticalMenu>
 <DdMenuItem label="Ro">
 </VerticalMenu>  这样才能正常！ #无法拆开，必须在同一个组件之内装配按钮根目录/树杈项按钮菜单。
 * */
export const VerticalMenu = (props) => {
    return (_jsx(DdMenu, { label: "\u83DC\u5355", tight: true, icon: _jsx(MenuItem, { variant: "ghost", component: "div", children: _jsx(IconMoreVertical, {}) }), ...props }));
};
//# sourceMappingURL=TwoHalfRightPanel.js.map